import joystick
import emc
import sys

BUTTON, AXIS = 1, 2

def tonumber(s):
    print "tonumber", repr(s)

    s = s.lower()
    if s in "xyzabc": return "xyzabc".index(s)
    try:
        return int(s)
    except ValueError:
        return float(s)

def set_nmlfile_from_inifile(inifile):
    ini = emc.ini(inifile)
    emc.nmlfile = ini.find("EMC", "NML_FILE")    

class JoystickApp:
    def __init__(self):
        self.current_axis = 0
        self.jogspeed = 0
        self.jog_cont = [False] * 6
        self.c = emc.command()
        self.s = emc.stat()
        self.bindings = {}

    def manual_ok(self):
        self.s.poll()
        if self.s.task_state != emc.STATE_ON: return False
        if self.s.interp_state != emc.INTERP_IDLE: return False
        if self.s.task_mode != emc.MODE_MANUAL:
            self.c.mode(emc.MODE_MANUAL)
        return True

    def ensure_mode(self, m):
        self.s.poll()
        if self.s.task_mode == m: return True
        if s.task_mode == emc.MODE_AUTO and s.interp_state != emc.INTERP_IDLE: \
            return False
        self.c.mode(m)
        return True

    def jogspeed_i(self):
        if self.jogspeed < .0002: self.jogspeed = .1
        else: self.jogspeed = self.jogspeed / 10

    def jogspeed_e(self, v):
        self.jogspeed = v

    def jogspeed_c(self):
        self.jogspeed = 0

    def activate_axis(self, i):
        self.jog_off(self.current_axis)
        self.current_axis = i

    def jog_on(self, a, b):
        if not self.manual_ok(): return
        print "jog_on", repr(self.jogspeed)
        if self.jogspeed != 0:
            self.jogspeed = float(self.jogspeed)
            self.c.jog(emc.JOG_INCREMENT, a, b, self.jogspeed)
            self.jog_cont[a] = False
        else:
            self.c.jog(emc.JOG_CONTINUOUS, a, b)
            self.jog_cont[a] = True

    def jog_on_current(self, b):
        self.jog_on(self.current_axis, b)

    def jog_off(self, a):
        if not self.manual_ok(): return
        if self.jog_cont[a]:
            self.c.jog(emc.JOG_STOP, a)
            self.jog_cont[a] = False

    def jog_off_current(self):
        self.jog_off(self.current_axis)

    def estop(self):
        self.s.poll()
        if self.s.task_state == emc.STATE_ESTOP: 
            self.c.state(emc.STATE_ESTOP_RESET)
        else:
            self.c.state(emc.STATE_ESTOP)

    def onoff(self):
        self.s.poll()
        if self.s.task_state == emc.STATE_ESTOP_RESET:
            self.c.state(emc.STATE_ON)
        else:
            self.c.state(emc.STATE_OFF)

    def run(self):
        self.ensure_mode(emc.MODE_AUTO)
        self.c.auto(emc.AUTO_RUN, 1)

    def step(self):
        self.ensure_mode(emc.MODE_AUTO)
        self.c.auto(emc.AUTO_STEP)

    def pause(self):
        self.ensure_mode(emc.MODE_AUTO)
        self.c.auto(emc.AUTO_PAUSE)

    def resume(self):
        self.ensure_mode(emc.MODE_AUTO)
        self.c.auto(emc.AUTO_RESUME)

    def pauseresume(self):
        self.ensure_mode(emc.MODE_AUTO)
        self.s.poll()
        if self.s.paused:
            self.c.auto(emc.AUTO_RESUME)
        else:
            self.c.auto(emc.AUTO_PAUSE)

    def _add_binding(self, type, index, rank, pred, command, args):
        command = getattr(self, command)
        bindings = self.bindings.setdefault((type, index), {})
        bindings[rank, pred] = command, args

    def _loadbindings(self, f):
        f = open(f)
        for line in f:
            line = line.split()
            if not line: continue

            kind = line[0]
            def is_one(x): return x==1
            def is_zero(x): return x==0
            def is_plus(x): return x>0
            def is_minus(x): return x<0
            def always(x): return True

            if kind == "ButtonPress":
                button = int(line[1])
                command = line[2]
                args = map(tonumber, line[3:])
                self._add_binding(BUTTON, button, 0, is_one, command, args)
            elif kind == "ButtonRelease":
                button = int(line[1])
                command = line[2]
                args = map(tonumber, line[3:])
                self._add_binding(BUTTON, button, 0, is_zero, command, args)
            elif kind == "AxisChange":
                axis = int(line[1])
                command = line[2]
                args = map(tonumber, line[3:])
                self._add_binding(AXIS, axis, 1, always, command, args)
            elif kind == "AxisZero":
                axis = int(line[1])
                command = line[2]
                args = map(tonumber, line[3:])
                self._add_binding(AXIS, axis, 0, is_zero, command, args)
            elif kind == "AxisMinus":
                axis = int(line[1])
                command = line[2]
                args = map(tonumber, line[3:])
                self._add_binding(AXIS, axis, 0, is_minus, command, args)
            elif kind == "AxisPlus":
                axis = int(line[1])
                command = line[2]
                args = map(tonumber, line[3:])
                self._add_binding(AXIS, axis, 0, is_plus, command, args)

    def _run(self):
        j = joystick.EventfulJoystick()
        while 1:
            time, value, type, number = j.get_next_event()
            print time, value, type, number

            b = self.bindings.get((type, number), None)
            if b is None: continue
            b = b.items()
            b.sort()
            print b
            for (rank, pred), (command, args) in b:
                if pred(value):
                    print command, args
                    command(*args)
                    break
                
if __name__ == '__main__':
    if len(sys.argv) > 1 and sys.argv[1] == '-ini':
        set_nmlfile_from_inifile(sys.argv[2])
        del sys.argv[1:3]
    j = JoystickApp()
    j._loadbindings(sys.argv[1])
    print j.bindings

    j._run()
